@@ -11,7 +11,7 @@ use crate::messaging::{system::LoadReport, WireMsg};
 use crate::routing::{
     error::{Error, Result},
     log_markers::LogMarker,
-    Peer,
+    Peer, UnnamedPeer,
 };
 use bytes::Bytes;
 use futures::{
@@ -86,7 +86,7 @@ impl Comm {
         bootstrap_nodes: &[SocketAddr],
         config: qp2p::Config,
         event_tx: mpsc::Sender<ConnectionEvent>,
-    ) -> Result<(Self, SocketAddr)> {
+    ) -> Result<(Self, UnnamedPeer)> {
         // Bootstrap to the network returning the connection to a node.
         // We can use the returned channels to listen for incoming messages and disconnection events
         let (endpoint, incoming_connections, bootstrap_peer) =
@@ -108,8 +108,7 @@ impl Comm {
 
         let _ = task::spawn(
             handle_incoming_messages(
-                bootstrap_peer.id(),
-                bootstrap_peer.remote_address(),
+                bootstrap_peer.clone(),
                 peer_incoming,
                 event_tx.clone(),
                 msg_count.clone(),
@@ -126,7 +125,7 @@ impl Comm {
                 back_pressure: BackPressure::new(),
                 connected_peers,
             },
-            bootstrap_peer.remote_address(),
+            UnnamedPeer::connected(bootstrap_peer),
         ))
     }
 
@@ -134,12 +133,6 @@ impl Comm {
         self.endpoint.public_addr()
     }
 
-    /// Get the connection ID (XorName) of an existing connection with the provided socket address
-    pub(crate) async fn get_peer_connection_id(&self, address: &SocketAddr) -> Option<XorName> {
-        let peer = self.connected_peers.get_by_address(address).await?;
-        Some(peer.id())
-    }
-
     /// Get the SocketAddr of a connection using the connection ID (XorName)
     pub(crate) async fn get_peer_address(&self, connection_id: &XorName) -> Option<SocketAddr> {
         let peer = self.connected_peers.get_by_id(connection_id).await?;
@@ -163,12 +156,19 @@ impl Comm {
             let priority = wire_msg.msg_kind().priority();
             let retries = self.back_pressure.get(&addr).await; // TODO: more laid back retries with lower priority, more aggressive with higher
 
-            self.connected_peers
-                .get_by_address(&addr)
-                .map(|res| res.ok_or(None))
-                .and_then(|client| async move {
-                    client
-                        .connection()
+            let connection = if let Some(connection) = recipient.connection().await {
+                Ok(connection)
+            } else {
+                self.connected_peers
+                    .get_by_address(&addr)
+                    .map(|res| res.ok_or(None))
+                    .map_ok(|client| client.connection().clone())
+                    .await
+            };
+
+            future::ready(connection)
+                .and_then(|connection| async move {
+                    connection
                         .send_with(bytes, priority, Some(&retries))
                         .await
                         .map_err(Some)
@@ -276,27 +276,50 @@ impl Comm {
 
                 let retries = self.back_pressure.get(&recipient.addr()).await; // TODO: more laid back retries with lower priority, more aggressive with higher
 
-                let existing_connection =
-                    self.connected_peers.get_by_address(&recipient.addr()).await;
-
-                let (connection, reused) = if let Some(connection) =
+                let (connection, reused) = if let Some(connection) = {
+                    if force_reconnection {
+                        None
+                    } else {
+                        recipient.connection().await
+                    }
+                } {
+                    trace!(
+                        connection_id = connection.id(),
+                        src = %connection.remote_address(),
+                        "{}",
+                        LogMarker::ConnectionReused
+                    );
+                    (Ok(connection), true)
+                } else if let Some(connection) = {
+                    let existing_connection =
+                        self.connected_peers.get_by_address(&recipient.addr()).await;
                     (!force_reconnection).then(|| existing_connection).flatten()
-                {
-                    (Ok(connection.connection().clone()), true)
+                } {
+                    let connection = connection.connection();
+                    trace!(
+                        connection_id = connection.id(),
+                        src = %connection.remote_address(),
+                        "{}",
+                        LogMarker::ConnectionReused
+                    );
+                    (Ok(connection.clone()), true)
                 } else {
                     (
                         self.endpoint
                             .connect_to(&recipient.addr())
                             .and_then(|(connection, connection_incoming)| async move {
+                                recipient.set_connection(connection.clone()).await;
                                 self.connected_peers.insert(connection.clone()).await;
-                                let _ = task::spawn(handle_incoming_messages(
-                                    connection.id(),
-                                    connection.remote_address(),
-                                    connection_incoming,
-                                    self.event_tx.clone(),
-                                    self.msg_count.clone(),
-                                    self.connected_peers.clone(),
-                                ));
+                                let _ = task::spawn(
+                                    handle_incoming_messages(
+                                        connection.clone(),
+                                        connection_incoming,
+                                        self.event_tx.clone(),
+                                        self.msg_count.clone(),
+                                        self.connected_peers.clone(),
+                                    )
+                                    .in_current_span(),
+                                );
                                 Ok(connection)
                             })
                             .await,
@@ -408,7 +431,7 @@ impl Comm {
 
 #[derive(Debug)]
 pub(crate) enum ConnectionEvent {
-    Received((SocketAddr, Bytes)),
+    Received((UnnamedPeer, Bytes)),
 }
 
 #[tracing::instrument(skip_all)]
@@ -419,14 +442,11 @@ async fn handle_incoming_connections(
     connected_peers: ConnectedPeers,
 ) {
     while let Some((connection, connection_incoming)) = incoming_connections.next().await {
-        let connection_id = connection.id();
-        let src = connection.remote_address();
-        connected_peers.insert(connection).await;
+        connected_peers.insert(connection.clone()).await;
 
         let _ = task::spawn(
             handle_incoming_messages(
-                connection_id,
-                src,
+                connection,
                 connection_incoming,
                 event_tx.clone(),
                 msg_count.clone(),
@@ -439,19 +459,25 @@ async fn handle_incoming_connections(
 
 #[tracing::instrument(skip(incoming_msgs, event_tx, msg_count, connected_peers))]
 async fn handle_incoming_messages(
-    connection_id: usize,
-    src: SocketAddr,
+    connection: qp2p::Connection,
     mut incoming_msgs: qp2p::ConnectionIncoming,
     event_tx: mpsc::Sender<ConnectionEvent>,
     msg_count: MsgCount,
     connected_peers: ConnectedPeers,
 ) {
+    let connection_id = connection.id();
+    let src = connection.remote_address();
     trace!(%connection_id, %src, "{}", LogMarker::ConnectionOpened);
 
     while let Some(result) = incoming_msgs.next().await.transpose() {
         match result {
             Ok(msg) => {
-                let _send_res = event_tx.send(ConnectionEvent::Received((src, msg))).await;
+                let _send_res = event_tx
+                    .send(ConnectionEvent::Received((
+                        UnnamedPeer::connected(connection.clone()),
+                        msg,
+                    )))
+                    .await;
                 // count incoming msgs..
                 msg_count.increase_incoming(src);
             }
@@ -712,6 +738,7 @@ mod tests {
         Ok(())
     }
 
+    #[cfg(not(feature = "unstable-no-connection-pooling"))]
     #[tokio::test(flavor = "multi_thread")]
     async fn connected_peers() -> Result<()> {
         let (node_tx, mut node_rx) = mpsc::channel(1);
@@ -736,7 +763,7 @@ mod tests {
         assert_matches!(node_rx.recv().await, Some(ConnectionEvent::Received(_)));
         assert!(
             node_comm
-                .get_peer_connection_id(&client_addr)
+                .get_peer_address(&ConnectedPeers::address_to_id(&client_addr))
                 .await
                 .is_some(),
             "did not find expected connection"
