@@ -7,33 +7,37 @@
 // permissions and limitations relating to use of the SAFE Network Software.
 
 use super::Core;
-use super::ProposalUtils;
 use crate::messaging::{
+    data::DataExchange,
     system::{
-        DkgSessionId, JoinResponse, Proposal, RelocateDetails, RelocatePromise, SectionAuth,
+        DkgSessionId, JoinResponse, NodeCmd, RelocateDetails, RelocatePromise, SectionAuth,
         SystemMsg,
     },
-    DstLocation, WireMsg,
+    DstLocation, MsgKind, WireMsg,
 };
 use crate::routing::{
-    core::StateSnapshot,
+    core::{Proposal, StateSnapshot},
     dkg::DkgSessionIdUtils,
-    error::Result,
+    error::{Error, Result},
     log_markers::LogMarker,
     messages::WireMsgUtils,
     network_knowledge::{ElderCandidates, NodeState, SectionKeyShare},
     relocation::RelocateState,
     routing_api::command::Command,
-    Peer,
+    Peer, UnnamedPeer,
 };
 use crate::types::PublicKey;
 use bls::PublicKey as BlsPublicKey;
-use xor_name::XorName;
+use xor_name::{Prefix, XorName};
 
 impl Core {
     // Send proposal to all our elders.
     pub(crate) async fn propose(&self, proposal: Proposal) -> Result<Vec<Command>> {
-        let elders: Vec<_> = self.network_knowledge.authority_provider().await.peers();
+        let elders = self
+            .network_knowledge
+            .authority_provider()
+            .await
+            .elders_vec();
         self.send_proposal(elders, proposal).await
     }
 
@@ -71,7 +75,7 @@ impl Core {
             recipients,
         );
 
-        let sig_share = proposal.prove(
+        let sig_share = proposal.sign_with_key_share(
             key_share.public_key_set.clone(),
             key_share.index,
             &key_share.secret_key_share,
@@ -79,8 +83,8 @@ impl Core {
 
         // Broadcast the proposal to the rest of the section elders.
         let node_msg = SystemMsg::Propose {
-            proposal,
-            sig_share,
+            proposal: proposal.clone().into_msg(),
+            sig_share: sig_share.clone(),
         };
         // Name of the section_pk may not matches the section prefix.
         // Carry out a substitution to prevent the dst_location becomes other section.
@@ -95,7 +99,34 @@ impl Core {
             section_key,
         )?;
 
-        Ok(self.send_or_handle(wire_msg, recipients).await)
+        let msg_id = wire_msg.msg_id();
+
+        let mut commands = vec![];
+        let our_name = self.node.read().await.name();
+        // handle ourselves if we should
+        for peer in recipients.clone() {
+            if peer.name() == our_name {
+                commands.extend(
+                    self.handle_proposal(msg_id, proposal.clone(), sig_share.clone(), peer)
+                        .await?,
+                )
+            }
+        }
+
+        // remove ourself from recipients
+        let recipients = recipients
+            .into_iter()
+            .filter(|peer| peer.name() != our_name)
+            .collect();
+
+        commands.extend(
+            self.send_messages_to_all_nodes_or_directly_handle_for_accumulation(
+                wire_msg, recipients,
+            )
+            .await?,
+        );
+
+        Ok(commands)
     }
 
     // ------------------------------------------------------------------------------------------------------------
@@ -105,7 +136,7 @@ impl Core {
     /// and members_info if required.
     pub(crate) async fn generate_ae_update(
         &self,
-        _dst_section_key: BlsPublicKey,
+        dst_section_key: BlsPublicKey,
         add_peer_info_to_update: bool,
     ) -> Result<SystemMsg> {
         let section_signed_auth = self
@@ -113,24 +144,20 @@ impl Core {
             .section_signed_authority_provider()
             .await
             .clone();
-        let section_auth = section_signed_auth.value;
+        let sap = section_signed_auth.value;
         let section_signed = section_signed_auth.sig;
 
-        let proof_chain = self.network_knowledge.chain().await;
-        /* TODO: get a proof chain rather than whole chain once we have a DAG for our chain.
         let proof_chain = match self
             .network_knowledge
-            .chain()
-            .await
             .get_proof_chain_to_current(&dst_section_key)
+            .await
         {
             Ok(chain) => chain,
             Err(_) => {
-                // error getting chain from key, so lets send the whole thing
-                self.network_knowledge.chain().await
+                // error getting chain from key, so let's send the whole thing
+                self.network_knowledge.section_chain().await
             }
         };
-        */
 
         let members = if add_peer_info_to_update {
             Some(
@@ -145,7 +172,7 @@ impl Core {
         };
 
         Ok(SystemMsg::AntiEntropyUpdate {
-            section_auth: section_auth.into_msg(),
+            section_auth: sap.to_msg(),
             section_signed,
             proof_chain,
             members,
@@ -157,7 +184,7 @@ impl Core {
         &self,
         node_state: SectionAuth<NodeState>,
     ) -> Vec<Command> {
-        let peer = node_state.to_peer();
+        let peer = node_state.peer().clone();
         info!(
             "Our section with {:?} has approved peer {}.",
             self.network_knowledge.prefix().await,
@@ -172,7 +199,7 @@ impl Core {
                 .await
                 .into_authed_msg(),
             node_state: node_state.into_authed_msg(),
-            section_chain: self.network_knowledge.chain().await,
+            section_chain: self.network_knowledge.section_chain().await,
         }));
 
         let dst_section_pk = self.network_knowledge.section_key().await;
@@ -220,7 +247,7 @@ impl Core {
         };
 
         match self
-            .send_direct_message_to_nodes(
+            .send_direct_message_to_nodes_in_section(
                 nodes,
                 node_msg,
                 self.network_knowledge.prefix().await.name(),
@@ -237,76 +264,82 @@ impl Core {
     }
 
     #[instrument(skip_all)]
-    pub(crate) async fn send_ae_update_to_sibling_section(
+    pub(crate) async fn send_updates_to_sibling_section(
         &self,
         old: &StateSnapshot,
-    ) -> Vec<Command> {
+    ) -> Result<Vec<Command>> {
         debug!("{}", LogMarker::AeSendUpdateToSiblings);
-        if let Some(sibling_sec_auth) = self
+        let mut commands = vec![];
+        if let Some(sibling_sap) = self
             .network_knowledge
             .prefix_map()
             .get_signed(&self.network_knowledge.prefix().await.sibling())
         {
-            let promoted_sibling_elders: Vec<_> = sibling_sec_auth
-                .peers()
-                .into_iter()
+            let promoted_sibling_elders: Vec<_> = sibling_sap
+                .elders()
                 .filter(|peer| !old.elders.contains(&peer.name()))
+                .cloned()
                 .collect();
 
             if promoted_sibling_elders.is_empty() {
                 debug!("No promoted siblings found in our network knowledge to send AE-Update");
-                return vec![];
+                return Ok(vec![]);
             }
 
-            // Using previous_key as dst_section_key as newly promoted sibling elders shall still
-            // in the state of pre-split.
-            let previous_pk = sibling_sec_auth.sig.public_key;
+            // Using previous_key as dst_section_key as newly promoted
+            // sibling elders shall still in the state of pre-split.
+            let previous_section_key = old.section_key;
 
-            // Compose a min sibling proof_chain.
-
-            /* TODO: get a proof chain rather than whole chain once we have a DAG for our chain.
-            let mut proof_chain = SecuredLinkedList::new(previous_pk);
-            */
-            let mut proof_chain = self.network_knowledge.chain().await;
-
-            let _ = proof_chain.insert(
-                &previous_pk,
-                sibling_sec_auth.section_key(),
-                sibling_sec_auth.sig.signature.clone(),
+            commands.extend(
+                self.send_data_updates_to(
+                    sibling_sap.prefix(),
+                    promoted_sibling_elders,
+                    previous_section_key,
+                )
+                .await?,
             );
 
-            let dst_name = sibling_sec_auth.prefix().name();
-
-            // Those promoted elders shall already know about other adult members.
-            // TODO: confirm no need to populate the members.
-            let node_msg = SystemMsg::AntiEntropyUpdate {
-                section_signed: sibling_sec_auth.sig,
-                section_auth: sibling_sec_auth.value.into_msg(),
-                proof_chain,
-                members: None,
-            };
-
-            match self
-                .send_direct_message_to_nodes(
-                    promoted_sibling_elders,
-                    node_msg,
-                    dst_name,
-                    previous_pk,
-                )
-                .await
-            {
-                Ok(cmd) => vec![cmd],
-                Err(err) => {
-                    error!(
-                        "Failed to send AE update to our promoted sibling elders: {:?}",
-                        err
-                    );
-                    vec![]
-                }
-            }
+            Ok(commands)
         } else {
             error!("Failed to get sibling SAP during split.");
-            vec![]
+            Ok(vec![])
+        }
+    }
+
+    /// Send DataExchange packet to the target recipients
+    pub(crate) async fn send_data_updates_to(
+        &self,
+        prefix: Prefix,
+        recipients: Vec<Peer>,
+        target_pk: BlsPublicKey,
+    ) -> Result<Vec<Command>> {
+        let chunk_data = self.get_data_of(&prefix).await;
+        let reg_data = self.register_storage.get_data_of(prefix).await?;
+
+        let data_update_msg = SystemMsg::NodeCmd(NodeCmd::ReceiveExistingData {
+            metadata: DataExchange {
+                chunk_data,
+                reg_data,
+            },
+        });
+
+        match self
+            .send_direct_message_to_nodes_in_section(
+                recipients.clone(),
+                data_update_msg,
+                prefix.name(),
+                target_pk,
+            )
+            .await
+        {
+            Ok(cmd) => Ok(vec![cmd]),
+            Err(err) => {
+                error!(
+                    "Failed to send data updates to: {:?} with {:?}",
+                    recipients, err
+                );
+                Ok(vec![])
+            }
         }
     }
 
@@ -326,7 +359,7 @@ impl Core {
         };
 
         match self
-            .send_direct_message_to_nodes(
+            .send_direct_message_to_nodes_in_section(
                 adults,
                 node_msg,
                 self.network_knowledge.prefix().await.name(),
@@ -336,10 +369,7 @@ impl Core {
         {
             Ok(cmd) => vec![cmd],
             Err(err) => {
-                error!(
-                    "Failed to send AE update to our promoted sibling elders: {:?}",
-                    err
-                );
+                error!("Failed to send AE update to our adults: {:?}", err);
                 vec![]
             }
         }
@@ -459,24 +489,32 @@ impl Core {
             recipients
         );
 
-        Ok(self.send_or_handle(wire_msg, recipients).await)
+        Ok(self
+            .send_messages_to_all_nodes_or_directly_handle_for_accumulation(wire_msg, recipients)
+            .await?)
     }
 
     // Send the message to all `recipients`. If one of the recipients is us, don't send it over the
-    // network but handle it directly.
-    pub(crate) async fn send_or_handle(
+    // network but handle it directly (should only be used when accumulation is necesary)
+    pub(crate) async fn send_messages_to_all_nodes_or_directly_handle_for_accumulation(
         &self,
         mut wire_msg: WireMsg,
         recipients: Vec<Peer>,
-    ) -> Vec<Command> {
+    ) -> Result<Vec<Command>> {
         let mut commands = vec![];
         let mut others = Vec::new();
         let mut handle = false;
 
         trace!("Send {:?} to {:?}", wire_msg, recipients);
 
-        for recipient in recipients {
+        for recipient in recipients.clone() {
             if recipient.name() == self.node.read().await.name() {
+                match *wire_msg.msg_kind() {
+                    MsgKind::NodeBlsShareAuthMsg(_) => {
+                        // do nothing, continue we should be accumulating this
+                    }
+                    _ => return Err(Error::SendOrHandlingNormalMsg),
+                };
                 handle = true;
             } else {
                 others.push(recipient);
@@ -499,13 +537,13 @@ impl Core {
             wire_msg.set_dst_xorname(self.node.read().await.name());
 
             commands.push(Command::HandleMessage {
-                sender_addr: self.node.read().await.addr,
+                sender: UnnamedPeer::addressed(self.our_connection_info()),
                 wire_msg,
                 original_bytes: None,
             });
         }
 
-        commands
+        Ok(commands)
     }
 
     pub(crate) async fn send_direct_message(
@@ -535,17 +573,17 @@ impl Core {
         })
     }
 
-    pub(crate) async fn send_direct_message_to_nodes(
+    pub(crate) async fn send_direct_message_to_nodes_in_section(
         &self,
         recipients: Vec<Peer>,
         node_msg: SystemMsg,
-        dst_name: XorName,
+        section_name: XorName,
         dst_section_pk: BlsPublicKey,
     ) -> Result<Command> {
         let wire_msg = WireMsg::single_src(
             &self.node.read().await.clone(),
             DstLocation::Section {
-                name: dst_name,
+                name: section_name,
                 section_pk: dst_section_pk,
             },
             node_msg,
@@ -566,11 +604,15 @@ impl Core {
     // TODO: consider changing this so it sends only to a subset of the elders
     // (say 1/3 of the ones closest to our name or so)
     pub(crate) async fn send_message_to_our_elders(&self, node_msg: SystemMsg) -> Result<Command> {
-        let targets: Vec<_> = self.network_knowledge.authority_provider().await.peers();
+        let targets = self
+            .network_knowledge
+            .authority_provider()
+            .await
+            .elders_vec();
 
         let dst_section_pk = self.network_knowledge.section_key().await;
         let cmd = self
-            .send_direct_message_to_nodes(
+            .send_direct_message_to_nodes_in_section(
                 targets,
                 node_msg,
                 self.network_knowledge
