@@ -26,16 +26,16 @@ mod split_barrier;
 
 pub(crate) use back_pressure::BackPressure;
 pub(crate) use bootstrap::{join_network, JoiningAsRelocated};
-pub(crate) use capacity::{CHUNK_COPY_COUNT, MIN_LEVEL_WHEN_FULL};
+pub(crate) use capacity::MIN_LEVEL_WHEN_FULL;
 pub(crate) use chunk_store::ChunkStore;
 pub(crate) use comm::{Comm, ConnectionEvent, SendStatus};
-pub(crate) use proposal::ProposalUtils;
+pub(crate) use proposal::Proposal;
 pub(crate) use register_storage::RegisterStorage;
 
 use self::split_barrier::SplitBarrier;
 use crate::dbs::UsedSpace;
+use crate::messaging::signature_aggregator::SignatureAggregator;
 use crate::messaging::system::SystemMsg;
-use crate::messaging::{signature_aggregator::SignatureAggregator, system::Proposal};
 use crate::routing::{
     dkg::DkgVoter,
     error::Result,
@@ -46,7 +46,7 @@ use crate::routing::{
     routing_api::command::Command,
     Elders, Event, NodeElderChange, Peer,
 };
-use crate::types::utils::write_data_to_disk;
+use crate::types::{utils::write_data_to_disk, Cache};
 use backoff::ExponentialBackoff;
 use capacity::Capacity;
 use itertools::Itertools;
@@ -56,6 +56,7 @@ use std::{
     collections::{BTreeMap, BTreeSet},
     path::PathBuf,
     sync::Arc,
+    time::Duration,
 };
 use tokio::sync::{mpsc, RwLock, Semaphore};
 use uluru::LRUCache;
@@ -67,6 +68,16 @@ pub(super) const RESOURCE_PROOF_DIFFICULTY: u8 = 10;
 const BACKOFF_CACHE_LIMIT: usize = 100;
 pub(crate) const CONCURRENT_JOINS: usize = 7;
 
+// How long to hold on to correlated `Peer`s for chunk queries. Since chunk queries are forwarded
+// from elders (with whom the client is connected) to adults (who hold the data), the elder handling
+// the query cannot reply immediately. For now, they stash a reference to the client `Peer` in
+// `Core::pending_chunk_queries`, which is a cache with duration-based expiry.
+// TODO: The value chosen here is longer than the default client timeout (see
+// `crate::client::SN_CLIENT_QUERY_TIMEOUT`), but the timeout is configurable. Ideally this would be
+// based on liveness properties (e.g. the timeout should be dynamic based on the responsiveness of
+// the section).
+const CHUNK_QUERY_TIMEOUT: Duration = Duration::from_secs(60 * 5 /* 5 mins */);
+
 // store up to 100 in use backoffs
 pub(crate) type AeBackoffCache =
     Arc<RwLock<LRUCache<(Peer, ExponentialBackoff), BACKOFF_CACHE_LIMIT>>>;
@@ -93,6 +104,7 @@ pub(crate) struct Core {
     root_storage_dir: PathBuf,
     capacity: Capacity,
     liveness: Liveness,
+    pending_chunk_queries: Arc<Cache<XorName, Peer>>,
     ae_backoff_cache: AeBackoffCache,
 }
 
@@ -140,6 +152,7 @@ impl Core {
             capacity,
             liveness: adult_liveness,
             root_storage_dir,
+            pending_chunk_queries: Arc::new(Cache::with_expiry_duration(CHUNK_QUERY_TIMEOUT)),
             ae_backoff_cache: AeBackoffCache::default(),
         })
     }
@@ -171,7 +184,7 @@ impl Core {
         let message = SystemMsg::AntiEntropyProbe(dst);
         let section_key = matching_section.section_key();
         let dst_name = matching_section.prefix().name();
-        let recipients = matching_section.peers();
+        let recipients = matching_section.elders_vec();
 
         info!(
             "ProbeMessage target {:?} w/key {:?}",
@@ -179,7 +192,7 @@ impl Core {
             section_key
         );
 
-        self.send_direct_message_to_nodes(recipients, message, dst_name, section_key)
+        self.send_direct_message_to_nodes_in_section(recipients, message, dst_name, section_key)
             .await
     }
 
@@ -189,7 +202,7 @@ impl Core {
 
         // Write to the node's root dir
         if let Err(e) = write_data_to_disk(
-            &self.network_knowledge.prefix_map(),
+            self.network_knowledge.prefix_map(),
             &self.root_storage_dir.join("prefix_map"),
         )
         .await
@@ -203,7 +216,7 @@ impl Core {
                 safe_dir.push(".safe");
                 safe_dir.push("prefix_map");
                 if let Err(e) =
-                    write_data_to_disk(&self.network_knowledge.prefix_map(), &safe_dir).await
+                    write_data_to_disk(self.network_knowledge.prefix_map(), &safe_dir).await
                 {
                     error!("Error writing PrefixMap to `~/.safe` dir: {:?}", e);
                 }
@@ -230,8 +243,7 @@ impl Core {
                     self.network_knowledge
                         .authority_provider()
                         .await
-                        .peers()
-                        .iter()
+                        .elders()
                         .format(", ")
                 );
 
@@ -278,46 +290,66 @@ impl Core {
                 NodeElderChange::None
             };
 
-            let sibling_elders = if new.prefix != old.prefix {
-                trace!("{}", LogMarker::NewPrefix);
-
-                self.network_knowledge
-                    .get_sap(&new.prefix.sibling())
-                    .map(|sap| {
-                        let current: BTreeSet<_> = sap.names();
-                        let added = current.difference(&old.elders).copied().collect();
-                        let removed = old.elders.difference(&current).copied().collect();
-                        let remaining = old.elders.intersection(&current).copied().collect();
-                        Elders {
-                            prefix: new.prefix.sibling(),
-                            key: sap.section_key(),
-                            remaining,
-                            added,
-                            removed,
-                        }
-                    })
-            } else {
-                None
-            };
-
-            let event = if let Some(sibling_elders) = sibling_elders {
+            // During the split, sibling's SAP could be unknown to us yet.
+            // Hence, fire the SectionSplit event whenever detect a prefix change.
+            // We also need to update other nodes w/ our known data.
+            let event = if (new.prefix != old.prefix) && new.is_elder {
                 info!("{}: {:?}", LogMarker::SplitSuccess, new.prefix);
-                // In case of split, send AE-Update to sibling new elder nodes.
-                commands.extend(self.send_ae_update_to_sibling_section(&old).await);
+
+                if old.is_elder {
+                    info!("{}: {:?}", LogMarker::StillElderAfterAplit, new.prefix);
+                }
+
+                commands.extend(self.send_updates_to_sibling_section(&old).await?);
+                self.retain_members_only(
+                    self.network_knowledge
+                        .adults()
+                        .await
+                        .iter()
+                        .map(|peer| peer.name())
+                        .collect(),
+                )
+                .await?;
 
                 Event::SectionSplit {
                     elders,
-                    sibling_elders,
                     self_status_change,
                 }
             } else {
+                commands.extend(
+                    self.send_data_updates_to(
+                        self.network_knowledge.prefix().await,
+                        self.network_knowledge
+                            .authority_provider()
+                            .await
+                            .elders_vec(),
+                        old.section_key,
+                    )
+                    .await?,
+                );
+
                 Event::EldersChanged {
                     elders,
                     self_status_change,
                 }
             };
 
-            self.send_event(event).await;
+            commands.extend(
+                self.send_data_updates_to(
+                    self.network_knowledge.prefix().await,
+                    self.network_knowledge
+                        .authority_provider()
+                        .await
+                        .elders()
+                        .filter(|peer| !old.elders.contains(&peer.name()))
+                        .cloned()
+                        .collect(),
+                    old.section_key,
+                )
+                .await?,
+            );
+
+            self.send_event(event).await
         }
 
         if !new.is_elder {
@@ -344,7 +376,7 @@ impl Core {
             // In case this assumption is not correct (because we already progressed more than one
             // key since the split) then this key would be unknown to them and they would send
             // us back their whole section chain. However, this situation should be rare.
-            *self.network_knowledge.chain().await.prev_key()
+            *self.network_knowledge.section_chain().await.prev_key()
         } else {
             *self.network_knowledge.genesis_key()
         }
@@ -357,6 +389,17 @@ impl Core {
             .print();
         self.comm.print_stats();
     }
+
+    pub(super) async fn log_network_stats(&self) {
+        let adults = self.network_knowledge.adults().await.len();
+        let elders = self
+            .network_knowledge
+            .authority_provider()
+            .await
+            .elder_count();
+        let prefix = self.network_knowledge.prefix().await;
+        debug!("{:?}: {:?} Elders, {:?} Adults.", prefix, elders, adults);
+    }
 }
 
 pub(crate) struct StateSnapshot {
