@@ -16,6 +16,7 @@ pub enum LogMarker {
     // stats
     TotalNetworkElders,
     // split
+    StillElderAfterAplit,
     SplitSuccess,
     SplitAttempt,
     NewPrefix,
@@ -57,7 +58,6 @@ pub enum LogMarker {
     // ae
     AeResendAfterRetry,
     AeResendAfterAeRedirect,
-    BounceAfterNewElderNotKnownLocally,
     AeSendRedirect,
     AeSendRetryAsOutdated,
     AeSendRetryDstPkFail,
@@ -80,4 +80,5 @@ pub enum LogMarker {
     // Connections
     ConnectionOpened,
     ConnectionClosed,
+    ConnectionReused,
 }
