@@ -6,14 +6,9 @@
 // KIND, either express or implied. Please review the Licences for the specific language governing
 // permissions and limitations relating to use of the SAFE Network Software.
 
-use crate::messaging::{system::Peer, NodeMsgAuthority, SrcLocation};
-use crate::routing::{
-    ed25519::{self},
-    error::{Error, Result},
-    peer::PeerUtils,
-};
+use crate::messaging::{NodeMsgAuthority, SrcLocation};
+use crate::routing::ed25519::{self};
 use bls::PublicKey as BlsPublicKey;
-use std::net::SocketAddr;
 use xor_name::XorName;
 
 pub(crate) trait NodeMsgAuthorityUtils {
@@ -21,9 +16,6 @@ pub(crate) trait NodeMsgAuthorityUtils {
 
     fn name(&self) -> XorName;
 
-    // If this location is `Node`, returns the corresponding `Peer` with `addr`. Otherwise error.
-    fn peer(&self, addr: SocketAddr) -> Result<Peer>;
-
     // Verify if the section key of the NodeMsgAuthority can be trusted
     // based on a set of known keys.
     fn verify_src_section_key_is_known(&self, known_keys: &[BlsPublicKey]) -> bool;
@@ -55,18 +47,6 @@ impl NodeMsgAuthorityUtils for NodeMsgAuthority {
         }
     }
 
-    // If this location is `Node`, returns the corresponding `Peer` with `addr`. Otherwise error.
-    fn peer(&self, addr: SocketAddr) -> Result<Peer> {
-        match self {
-            NodeMsgAuthority::Node(node_auth) => {
-                Ok(Peer::new(ed25519::name(&node_auth.public_key), addr))
-            }
-            NodeMsgAuthority::Section(_) | NodeMsgAuthority::BlsShare(_) => {
-                Err(Error::InvalidSrcLocation)
-            }
-        }
-    }
-
     // Verify if it's a section/bls-share signed authority,
     // and if can be trusted based on a set of known keys.
     fn verify_src_section_key_is_known(&self, known_keys: &[BlsPublicKey]) -> bool {
