@@ -24,10 +24,10 @@
 // Public API
 // ############################################################################
 pub(crate) use self::{
-    core::ChunkStore,
-    core::RegisterStorage,
-    core::{CHUNK_COPY_COUNT, MIN_LEVEL_WHEN_FULL},
-    network_knowledge::{section_keys::SectionKeyShare, SectionAuthorityProvider},
+    core::MIN_LEVEL_WHEN_FULL,
+    network_knowledge::{
+        peer::UnnamedPeer, section_keys::SectionKeyShare, SectionAuthorityProvider,
+    },
 };
 pub use self::{
     dkg::SectionAuthUtils,
@@ -75,10 +75,9 @@ mod routing_api;
 /// More nodes might be added if requested by the upper layers.
 /// This number also detemines when split happens - if both post-split sections would have at least
 /// this number of nodes.
-pub const RECOMMENDED_SECTION_SIZE: usize = 2 * ELDER_SIZE;
-
-/// Number of elders per section.
-pub const ELDER_SIZE: usize = 7;
+pub(crate) fn recommended_section_size() -> usize {
+    2 * crate::elder_count()
+}
 
 /// SuperMajority of a given group (i.e. > 2/3)
 #[inline]
